@@ -24,18 +24,21 @@
 
 #include "providermanager.h"
 
+#include "attica_debug.h"
+#include "atticautils.h"
+
 #include <QCoreApplication>
-#include <QtCore/QDebug>
-#include <QtCore/QFile>
-#include <QtCore/QPluginLoader>
-#include <QtCore/QSet>
-#include <QtCore/QSignalMapper>
-#include <QtCore/QTimer>
-#include <QtCore/QProcess>
-#include <QtNetwork/QAuthenticator>
-#include <QtNetwork/QNetworkReply>
-#include <QtNetwork/QNetworkProxy>
-#include <QtCore/QXmlStreamReader>
+#include <QDebug>
+#include <QFile>
+#include <QPluginLoader>
+#include <QSet>
+#include <QTimer>
+#include <QProcess>
+#include <QAuthenticator>
+#include <QNetworkReply>
+#include <QNetworkProxy>
+#include <QXmlStreamReader>
+#include <QSignalMapper>
 
 #include "platformdependent.h"
 #include "qtplatformdependent_p.h"
@@ -49,7 +52,6 @@ public:
     PlatformDependent *m_internals;
     QHash<QUrl, Provider> m_providers;
     QHash<QUrl, QUrl> m_providerTargets;
-    QSignalMapper m_downloadMapping;
     QHash<QString, QNetworkReply *> m_downloads;
     bool m_authenticationSuppressed;
 
@@ -80,13 +82,12 @@ ProviderManager::ProviderManager(const ProviderFlags &flags)
     : d(new Private)
 {
     d->m_internals = loadPlatformDependent(flags);
-    connect(d->m_internals->nam(), SIGNAL(authenticationRequired(QNetworkReply*,QAuthenticator*)), SLOT(authenticate(QNetworkReply*,QAuthenticator*)));
-    connect(&d->m_downloadMapping, SIGNAL(mapped(QString)), SLOT(fileFinished(QString)));
+    connect(d->m_internals->nam(), &QNetworkAccessManager::authenticationRequired, this, &ProviderManager::authenticate);
 }
 
 void ProviderManager::loadDefaultProviders()
 {
-    QTimer::singleShot(0, this, SLOT(slotLoadDefaultProvidersInternal()));
+    QTimer::singleShot(0, this, &ProviderManager::slotLoadDefaultProvidersInternal);
 }
 
 void ProviderManager::setAuthenticationSuppressed(bool suppressed)
@@ -102,7 +103,8 @@ void ProviderManager::clear()
 
 void ProviderManager::slotLoadDefaultProvidersInternal()
 {
-    foreach (const QUrl &url, d->m_internals->getDefaultProviderFiles()) {
+    const auto providerFiles  = d->m_internals->getDefaultProviderFiles();
+    for (const QUrl &url : providerFiles) {
         addProviderFile(url);
     }
     if (d->m_downloads.isEmpty()) {
@@ -143,12 +145,12 @@ void ProviderManager::addProviderFile(const QUrl &url)
     } else {
         if (!d->m_downloads.contains(url.toString())) {
             QNetworkRequest req(url);
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 6, 0))
             req.setAttribute(QNetworkRequest::FollowRedirectsAttribute, true);
-#endif
             QNetworkReply *reply = d->m_internals->get(req);
-            connect(reply, SIGNAL(finished()), &d->m_downloadMapping, SLOT(map()));
-            d->m_downloadMapping.setMapping(reply, url.toString());
+            qCDebug(ATTICA) << "executing" << Utils::toString(reply->operation()) << "for" << reply->url();
+            connect(reply, &QNetworkReply::finished, this, [this, url]() {
+                fileFinished(url.toString());
+            });
             d->m_downloads.insert(url.toString(), reply);
         }
     }
@@ -157,7 +159,10 @@ void ProviderManager::addProviderFile(const QUrl &url)
 void ProviderManager::fileFinished(const QString &url)
 {
     QNetworkReply *reply = d->m_downloads.take(url);
-    parseProviderFile(QLatin1String(reply->readAll()), QUrl(url));
+    if (reply->error())
+        Q_EMIT failedToLoad(QUrl(url), reply->error());
+    else
+        parseProviderFile(QLatin1String(reply->readAll()), QUrl(url));
     reply->deleteLater();
 }
 
@@ -225,7 +230,7 @@ void ProviderManager::parseProviderFile(const QString &xmlString, const QUrl &ur
                 }
             }
             if (!baseUrl.isEmpty()) {
-                //qDebug() << "Adding provider" << baseUrl;
+                //qCDebug(ATTICA) << "Adding provider" << baseUrl;
                 d->m_providers.insert(baseUrl, Provider(d->m_internals, baseUrl, name, icon,
                                                         person, friendV, message, achievement, activity, content, fan, forum, knowledgebase,
                                                         event, comment, registerUrl));
@@ -236,7 +241,7 @@ void ProviderManager::parseProviderFile(const QString &xmlString, const QUrl &ur
     }
 
     if (xml.error() != QXmlStreamReader::NoError) {
-        qDebug() << "error:" << xml.errorString() << "in" << url;
+        qCDebug(ATTICA) << "error:" << xml.errorString() << "in" << url;
     }
 
     if (d->m_downloads.isEmpty()) {
@@ -272,22 +277,22 @@ QList<QUrl> ProviderManager::providerFiles() const
 void ProviderManager::authenticate(QNetworkReply *reply, QAuthenticator *auth)
 {
     QUrl baseUrl;
-    QList<QUrl> urls = d->m_providers.keys();
-    foreach (const QUrl &url, urls) {
+    const QList<QUrl> urls = d->m_providers.keys();
+    for (const QUrl &url : urls) {
         if (url.isParentOf(reply->url())) {
             baseUrl = url;
             break;
         }
     }
 
-    //qDebug() << "ProviderManager::authenticate" << baseUrl;
+    //qCDebug(ATTICA) << "ProviderManager::authenticate" << baseUrl;
 
     QString user;
     QString password;
     if (auth->user().isEmpty() && auth->password().isEmpty()) {
         if (d->m_internals->hasCredentials(baseUrl)) {
             if (d->m_internals->loadCredentials(baseUrl, user, password)) {
-                //qDebug() << "ProviderManager::authenticate: loading authentication";
+                //qCDebug(ATTICA) << "ProviderManager::authenticate: loading authentication";
                 auth->setUser(user);
                 auth->setPassword(password);
                 return;
@@ -296,7 +301,7 @@ void ProviderManager::authenticate(QNetworkReply *reply, QAuthenticator *auth)
     }
 
     if (!d->m_authenticationSuppressed && d->m_internals->askForCredentials(baseUrl, user, password)) {
-        //qDebug() << "ProviderManager::authenticate: asking internals for new credentials";
+        //qCDebug(ATTICA) << "ProviderManager::authenticate: asking internals for new credentials";
         //auth->setUser(user);
         //auth->setPassword(password);
         return;
@@ -315,7 +320,7 @@ void ProviderManager::proxyAuthenticationRequired(const QNetworkProxy &proxy, QA
 
 void ProviderManager::initNetworkAccesssManager()
 {
-    connect(d->m_internals->nam(), SIGNAL(authenticationRequired(QNetworkReply*,QAuthenticator*)), this, SLOT(authenticate(QNetworkReply*,QAuthenticator*)));
-    connect(d->m_internals->nam(), SIGNAL(proxyAuthenticationRequired(QNetworkProxy,QAuthenticator*)), this, SLOT(proxyAuthenticationRequired(QNetworkProxy,QAuthenticator*)));
+    connect(d->m_internals->nam(), &QNetworkAccessManager::authenticationRequired, this, &ProviderManager::authenticate);
+    connect(d->m_internals->nam(), &QNetworkAccessManager::proxyAuthenticationRequired, this, &ProviderManager::proxyAuthenticationRequired);
 }
 
